"use strict";
var helpers = require('./helpers');
var originalEnv = process.env;
describe('helpers', function () {
    beforeEach(function () {
        originalEnv = process.env;
        process.env = {};
    });
    afterEach(function () {
        process.env = originalEnv;
    });
    describe('getIntPropertyValue', function () {
        it('should return an int', function () {
            // arrange
            var propertyName = 'test';
            var propertyValue = '3000';
            process.env[propertyName] = propertyValue;
            // act
            var result = helpers.getIntPropertyValue(propertyName);
            // assert
            expect(result).toEqual(3000);
        });
        it('should round to an int', function () {
            // arrange
            var propertyName = 'test';
            var propertyValue = '3000.03';
            process.env[propertyName] = propertyValue;
            // act
            var result = helpers.getIntPropertyValue(propertyName);
            // assert
            expect(result).toEqual(3000);
        });
        it('should round to a NaN', function () {
            // arrange
            var propertyName = 'test';
            var propertyValue = 'tacos';
            process.env[propertyName] = propertyValue;
            // act
            var result = helpers.getIntPropertyValue(propertyName);
            // assert
            expect(result).toEqual(NaN);
        });
    });
    describe('getBooleanPropertyValue', function () {
        it('should return true when value is "true"', function () {
            // arrange
            var propertyName = 'test';
            var propertyValue = 'true';
            process.env[propertyName] = propertyValue;
            // act
            var result = helpers.getBooleanPropertyValue(propertyName);
            // assert
            expect(result).toEqual(true);
        });
        it('should return false when value is undefined/null', function () {
            // arrange
            var propertyName = 'test';
            // act
            var result = helpers.getBooleanPropertyValue(propertyName);
            // assert
            expect(result).toEqual(false);
        });
        it('should return false when value is not "true"', function () {
            // arrange
            var propertyName = 'test';
            var propertyValue = 'taco';
            process.env[propertyName] = propertyValue;
            // act
            var result = helpers.getBooleanPropertyValue(propertyName);
            // assert
            expect(result).toEqual(false);
        });
    });
    describe('processStatsImpl', function () {
        it('should convert object graph to known module map', function () {
            // arrange
            var moduleOne = '/Users/dan/myModuleOne.js';
            var moduleTwo = '/Users/dan/myModuleTwo.js';
            var moduleThree = '/Users/dan/myModuleThree.js';
            var moduleFour = '/Users/dan/myModuleFour.js';
            var objectGraph = {
                modules: [
                    {
                        identifier: moduleOne,
                        reasons: [
                            {
                                moduleIdentifier: moduleTwo
                            },
                            {
                                moduleIdentifier: moduleThree
                            }
                        ]
                    },
                    {
                        identifier: moduleTwo,
                        reasons: [
                            {
                                moduleIdentifier: moduleThree
                            }
                        ]
                    },
                    {
                        identifier: moduleThree,
                        reasons: [
                            {
                                moduleIdentifier: moduleOne
                            }
                        ]
                    },
                    {
                        identifier: moduleFour,
                        reasons: []
                    }
                ]
            };
            // act
            var result = helpers.processStatsImpl(objectGraph);
            // assert
            var setOne = result.get(moduleOne);
            expect(setOne.has(moduleTwo)).toBeTruthy();
            expect(setOne.has(moduleThree)).toBeTruthy();
            var setTwo = result.get(moduleTwo);
            expect(setTwo.has(moduleThree)).toBeTruthy();
            var setThree = result.get(moduleThree);
            expect(setThree.has(moduleOne)).toBeTruthy();
            var setFour = result.get(moduleFour);
            expect(setFour.size).toEqual(0);
        });
    });
});
